---
description: ''
---

# Typescript

There are diffrent ways to write the classes and having typescript types. I want to show you here the diffrent approches.

## Decorators with declare (recommanded)

::alert{type="info"}
The tsconfig
```json
...
"useDefineForClassFields": true,  // Default true if target is "ES2022" or "ESNext"
"experimentalDecorators": true,
...
```
::

This is pretty new if you are a vuex-orm user. If you are a vuex-orm-next user than you are already used to it.

````ts
import { Model } from 'pinia-orm'
import { Attr, Cast, Uid } form 'pinia-orm/dist/decorators'
import { ArrayCast } from 'pinia-orm/dist/casts'

class User extends Model {
  static entity = 'users'

  @Uid() declare id: string
  @Cast(() => ArrayCast) @Attr('{}') declare meta: Record<string, any>
}
````

For more information about using `declare` read this [ticket](https://github.com/CodeDredd/pinia-orm/issues/148) 

## Without declare

### With decorators

````ts
import { Model } from 'pinia-orm'
import { Attr, Cast, Uid } form 'pinia-orm/dist/decorators'
import { ArrayCast } from 'pinia-orm/dist/casts'

class User extends Model {
  static entity = 'users'

  @Uid() id!: string
  @Cast(() => ArrayCast) @Attr('{}') meta!: Record<string, any>
}
````

::alert{type="info"}
#### **Vite Integration**

Make sure to disable `useDefineForClassFields` in `tsconfig.json` when using `vite >= 2.5.0`. See [this issue](https://github.com/vitejs/vite/issues/4636) for more details.

```json
...
"useDefineForClassFields": false,
...
```
::

### without decorators (used way from vuex-orm)



````ts
import { Model } from 'pinia-orm'
import { ArrayCast } from 'pinia-orm/dist/casts'

class User extends Model {
  static entity = 'users'

  static fields() {
    return {
      id: this.uid(),
      meta: this.attr('{}')
    }
  }

  static casts() {
    return {
      meta: ArrayCast,
    }
  }

  id!: number
  meta!: Record<string, any>
}
````
